@@ -61,6 +61,7 @@ import javax.ws.rs.Produces;
 import javax.ws.rs.core.Context;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
+import javax.ws.rs.core.Response.ResponseBuilder;
 
 import com.openexchange.java.Strings;
 import com.openexchange.mobile.api.facade.auth.AuthenticationService;
@@ -78,6 +79,7 @@ import com.openexchange.mobile.api.facade.services.ConfigService;
 import com.openexchange.mobile.api.facade.utils.CookieUtil;
 import com.openexchange.mobile.api.facade.utils.JacksonUtil;
 import com.openexchange.mobile.api.facade.utils.RequestUtil;
+import com.openexchange.mobile.api.facade.utils.UserAgentInfo;
 
 import lombok.RequiredArgsConstructor;
 import lombok.SneakyThrows;
@@ -110,14 +112,18 @@ public class AuthenticationEndpoint {
         AuthenticationRequestBody authenticationRequestBody = JacksonUtil.parse(rawRequestBody, AuthenticationRequestBody.class);
         validate(authenticationRequestBody);
         String hostname = RequestUtil.getHostHeader(request);
+        String xHostname = RequestUtil.getXHostHeader(request);
         String userAgent = RequestUtil.getUserAgent(request);
         String forwardedFor = RequestUtil.getForwardedForWithRemoteIp(request);
         String proto = RequestUtil.getForwardedProto(request);
         String trackingId = RequestUtil.getTrackingId(request);
-        HostConfiguration configuration = ConfigurationHelper.getConfiguration(hostname);
-        RequestUtil.checkUserAgent(userAgent, configuration);
+        UserAgentInfo userAgentInfo = RequestUtil.parseUserAgent(userAgent);
+        HostConfiguration configuration = ConfigurationHelper.getConfiguration(hostname, xHostname, userAgentInfo);
+        RequestUtil.checkUserAgent(userAgentInfo, configuration);
         LoginCredentials credentials = getCredentials(request, authenticationRequestBody)
+                .withHostConfiguration(configuration)
                 .withHost(hostname)
+                .withXHost(xHostname)
                 .withUserAgent(userAgent)
                 .withForwardedFor(forwardedFor)
                 .withForwardedProto(proto)
@@ -125,7 +131,7 @@ public class AuthenticationEndpoint {
         try {
             AuthenticationService authenticationService = authenticationHandler.getAuthenticationService(configuration);
             SessionData sessionData = authenticationService.authenticate(credentials);
-            sessionData = RequestUtil.enhanceSessionData(request, configuration, hostname, userAgent, null, null, null, sessionData);
+            sessionData = RequestUtil.enhanceSessionData(request, configuration, hostname, xHostname, userAgent, null, null, null, sessionData);
             sessionData = configService.addConfiguration(sessionData);
             return buildResponse(sessionData);
         } catch (ApiFacadeException e) {
@@ -195,13 +201,14 @@ public class AuthenticationEndpoint {
         String rawResponseBody = JacksonUtil.toString(responseBody);
         Response.ResponseBuilder builder = Response.ok(rawResponseBody);
         CookieUtil.addToResponse(builder, sessionData.getCookies());
-        return builder.build();
+        return RequestUtil.nonCachable(builder).build();
     }
 
     @SneakyThrows
     private Response buildRedirectResponse(ApiFacadeException e) {
         String baseUrl = e.getMessage();
-        return Response.temporaryRedirect(new URI(baseUrl)).build();
+        ResponseBuilder builder = Response.temporaryRedirect(new URI(baseUrl));
+        return RequestUtil.nonCachable(builder).build();
     }
 
 }
